@@ -26,9 +26,9 @@
 #' `".chain"`, `".iteration"`, `".draw"`, and `".row"`) will be summarized.
 #' This can be list columns.
 #' @param .along Which columns are the input values to the function describing the curve (e.g., the "x"
-#' values). Supports tidyselect syntax, as in `dplyr::select()`. Intervals are calculated jointly with
+#' values). Supports tidyselect syntax, as in [dplyr::select()]. Intervals are calculated jointly with
 #' respect to these variables, conditional on all other grouping variables in the data frame. The default
-#' (`NULL`) causes `curve_interval()` to use all grouping variables in the input data frame as the value
+#' (`NULL`) causes [curve_interval()] to use all grouping variables in the input data frame as the value
 #' for `.along`, which will generate the most conservative intervals. However, if you want to calculate
 #' intervals for some function `y = f(x)` conditional on some other variable(s) (say, conditional on a
 #' factor `g`), you would group by `g`, then use `.along = x` to calculate intervals jointly over `x`
@@ -87,7 +87,7 @@
 #' *arXiv e-print*.
 #' [arXiv:2007.05035](https://arxiv.org/abs/2007.05035)
 #'
-#' @seealso `point_interval()` for pointwise intervals. See `vignette("lineribbon")` for more examples
+#' @seealso [point_interval()] for pointwise intervals. See `vignette("lineribbon")` for more examples
 #' and discussion of the differences between pointwise and curvewise intervals.
 #' @examples
 #'
@@ -115,21 +115,20 @@
 #'   ggtitle("50% pointwise intervals with point_interval()") +
 #'   theme_ggdist()
 #'
+#' @examplesIf requireNamespace("posterior", quietly = TRUE)
 #' # ... compare them to curvewise intervals
-#' if (requireNamespace("posterior", quietly = TRUE)) {
-#'   df %>%
-#'     group_by(x) %>%
-#'     curve_interval(y, .width = c(.5)) %>%
-#'     ggplot(aes(x = x, y = y)) +
-#'     geom_lineribbon(aes(ymin = .lower, ymax = .upper)) +
-#'     geom_line(aes(group = .draw), alpha=0.15, data = df) +
-#'     scale_fill_brewer() +
-#'     ggtitle("50% curvewise intervals with curve_interval()") +
-#'     theme_ggdist()
-#' }
+#' df %>%
+#'   group_by(x) %>%
+#'   curve_interval(y, .width = c(.5)) %>%
+#'   ggplot(aes(x = x, y = y)) +
+#'   geom_lineribbon(aes(ymin = .lower, ymax = .upper)) +
+#'   geom_line(aes(group = .draw), alpha=0.15, data = df) +
+#'   scale_fill_brewer() +
+#'   ggtitle("50% curvewise intervals with curve_interval()") +
+#'   theme_ggdist()
 #'
 #' @importFrom dplyr group_vars summarise_at %>% group_split
-#' @importFrom rlang quos quos_auto_name eval_tidy quo_get_expr
+#' @importFrom rlang quos quos_auto_name eval_tidy quo_get_expr syms enquo
 #' @importFrom tidyselect eval_select
 #' @export
 curve_interval = function(.data, ..., .along = NULL, .width = .5,
@@ -137,7 +136,7 @@ curve_interval = function(.data, ..., .along = NULL, .width = .5,
   na.rm = FALSE, .exclude = c(".chain", ".iteration", ".draw", ".row")
 ) {
   if (!requireNamespace("posterior", quietly = TRUE)) {
-    stop('curve_interval() requires the `posterior` package to be installed.') #nocov
+    stop0('curve_interval() requires the `posterior` package to be installed.') #nocov
   }
 
   .interval = match.arg(.interval)
@@ -163,14 +162,12 @@ curve_interval = function(.data, ..., .along = NULL, .width = .5,
       #don't aggregate groups because we aggregate within these
       setdiff(group_vars(data)) %>%
       setdiff(.exclude) %>%
-      # have to use quos here because lists of symbols don't work correctly with iwalk() for some reason
-      # (the simpler version of this line would be `syms() %>%`)
-      lapply(function(x) quo(!!sym(x))) %>%
+      syms() %>%
       quos_auto_name()
 
     if (length(col_exprs) == 0) {
       #still nothing to aggregate? not sure what the user wants
-      stop("No columns found to calculate point and interval summaries for.")
+      stop0("No columns found to calculate point and interval summaries for.")
     }
   }
 
@@ -191,9 +188,9 @@ curve_interval = function(.data, ..., .along = NULL, .width = .5,
 
     .curve_interval(data, col_name, ".lower", ".upper", .width, .interval, .conditional_groups, na.rm = na.rm)
   } else {
-    iwalk_(col_exprs, function(col_expr, col_name) {
-      data[[col_name]] <<- eval_tidy(col_expr, data)
-    })
+    for (i in seq_along(col_exprs)) {
+      data[[names(col_exprs)[[i]]]] = eval_tidy(col_exprs[[i]], data)
+    }
 
     # if the values we are going to summarise are not already list columns, make them into list columns
     # (making them list columns first is faster than anything else I've tried)
@@ -201,13 +198,24 @@ curve_interval = function(.data, ..., .along = NULL, .width = .5,
       data = summarise_at(data, names(col_exprs), list)
     }
 
+    result = NULL
+    actual_widths = NULL
     for (col_name in names(col_exprs)) {
-      data = .curve_interval(
+      col_result = .curve_interval(
         data, col_name, paste0(col_name, ".lower"), paste0(col_name, ".upper"), .width, .interval, .conditional_groups, na.rm = na.rm
       )
-    }
 
-    data
+      # actual widths aren't always going to be equal so we'll take the means of them
+      actual_widths = cbind(actual_widths, col_result$.actual_width)
+
+      result = bind_cols(
+        result[, names(result) != col_name],
+        col_result[, names(col_result) == col_name | (!names(col_result) %in% names(result))]
+      )
+    }
+    result$.actual_width = rowMeans(actual_widths)
+
+    result
   }
 
   result[[".point"]] = .interval
@@ -224,7 +232,7 @@ halfspace_depth = function(x) {
 
 .curve_interval = function(data, col_name, lower, upper, .width, .interval, .conditional_groups, na.rm = FALSE) {
   if (length(unique(lengths(data[[col_name]]))) != 1) {
-    stop("Must have the same number of values in each group.")
+    stop0("Must have the same number of values in each group.")
   }
 
   dfs = group_split(group_by_at(data, .conditional_groups))
@@ -280,22 +288,24 @@ halfspace_depth = function(x) {
     # by the envelope around all draws deeper than the depth cutoff
     sorted_draw_depths = sort(draw_depth)
     map_dfr_(.width, function(w) {
-      if (FALSE) {
-        # naive approach: just use quantiles of draw depths to determine cutoff
-        depth_cutoff = quantile(draw_depth, 1 - w, na.rm = na.rm)
-      } else {
-        # use binary search to find a cutoff
-        draw_depth_i = binary_search(
-          function(i) {
-            depth_cutoff = sorted_draw_depths[i]
-            actual_width = calc_intervals_at_depth_cutoff(depth_cutoff)[[".actual_width"]][[1]]
-            actual_width >= w
-          },
-          min_i = 1,
-          max_i = length(sorted_draw_depths)
-        )
-        depth_cutoff = sorted_draw_depths[draw_depth_i]
-      }
+      # The naive approach would be to just use quantiles of draw depths to determine
+      # the cutoff; something like:
+      #   depth_cutoff = quantile(draw_depth, 1 - w, na.rm = na.rm)
+      # However this does not work well, since the envelope around a w% set of curves
+      # determined via quantiles tends to incidentally cover some other curves, making
+      # the coverage be (sometimes substantially) more than w%.
+      # Thus instead, we use binary search to find a depth cutoff that contains
+      # w% of the curves:
+      draw_depth_i = binary_search(
+        function(i) {
+          depth_cutoff = sorted_draw_depths[i]
+          actual_width = calc_intervals_at_depth_cutoff(depth_cutoff)[[".actual_width"]][[1]]
+          actual_width >= w
+        },
+        min_i = 1,
+        max_i = length(sorted_draw_depths)
+      )
+      depth_cutoff = sorted_draw_depths[draw_depth_i]
 
       d = calc_intervals_at_depth_cutoff(depth_cutoff)
       d[[col_name]] = median_y
