@@ -8,7 +8,7 @@ library(tidyr)
 library(purrr)
 library(distributional)
 
-context("scale_colour_ramp")
+
 
 
 test_that("basic fill_ramp works", {
@@ -121,7 +121,7 @@ test_that("fill_ramp works with stat_slab and NAs", {
             labels = scales::percent_format()
           ))
         ),
-        position = "dodge",
+        position = "dodgejust",
         n = 50
       ) +
       scale_fill_ramp_discrete(range = c(1, 0.2), na.translate = FALSE) +
